from collections import defaultdict

from match import StructureMatch
from representation_assigner import RepresentationAssigner
from progress_bar import progress

class MatchStore:
    def __init__(self, args, db):
        self.db = db
        self.data = {}
        self.min_frequency = args.min_freq
        self.dispersions = {}

        self.db.init("""CREATE TABLE Colocations (
            colocation_id INTEGER PRIMARY KEY,
            structure_id INTEGER,
            key varchar(256))
            """)
        self.db.init("""CREATE TABLE Matches (
            match_id INTEGER,
            component_id INTEGER NOT NULL,
            word_lemma varchar(32) NOT NULL,
            word_id varchar(32) NOT NULL,
            word_msd varchar(16) NOT NULL,
            word_text varchar(32) NOT NULL)
            """)
        self.db.init("""CREATE TABLE ColocationMatches (
            mid_match_id INTEGER,
            mid_colocation_id INTEGER,
            FOREIGN KEY(mid_colocation_id) REFERENCES Colocations(colocation_id),
            FOREIGN KEY(mid_match_id) REFERENCES Matches(match_id))
            """)
        self.db.init("""CREATE TABLE Representations (
            colocation_id INTEGER,
            component_id INTEGER,
            text varchar(32),
            FOREIGN KEY(colocation_id) REFERENCES Colocations(colocation_id))
            """)
        

    def _add_match(self, key, structure, match):
        structure_id, key_str = key[0], str(key[1:])
        cid = self.db.execute("SELECT colocation_id FROM Colocations WHERE key=? AND structure_id=?",
                              (key_str, structure_id)).fetchone()

        if cid is None:
            self.db.execute("INSERT INTO Colocations (structure_id, key) VALUES (?,?)", 
                            (structure_id, key_str))
            cid = self.db.execute("SELECT colocation_id FROM Colocations WHERE key=? AND structure_id=?",
                                (key_str, structure_id)).fetchone()
        
        mid = self.db.execute("SELECT max(match_id) + 1 FROM Matches").fetchone()
        mid = 0 if mid[0] is None else mid[0]

        for component_id, word in match.items():
            self.db.execute("""
            INSERT INTO Matches (match_id, component_id, word_lemma, word_text, word_msd, word_id) 
            VALUES (:match_id, :component_id, :word_lemma, :word_text, :word_msd, :word_id)""", {
                "component_id": component_id,
                "match_id": mid,
                "word_lemma": word.lemma,
                "word_msd": word.msd,
                "word_text": word.text,
                "word_id": word.id,
            })

        self.db.execute("INSERT INTO ColocationMatches (mid_colocation_id, mid_match_id) VALUES (?,?)",
            (cid[0], mid))

        if key not in self.data:
            self.data[key] = StructureMatch(str(len(self.data) + 1), structure)
        self.data[key].append(match)

    def add_matches(self, matches):
        for structure, nms in matches.items():
            for nm in nms:
                self._add_match(nm[1], structure, nm[0])

    def get_matches_for(self, structure):
        print(structure.id)
        for cid in self.db.execute("SELECT colocation_id FROM Colocations WHERE structure_id=?",
                                   (structure.id,)):
            yield StructureMatch.from_db(self.db, cid[0], structure)

        # for _cid_tup, sm in self.data.items():
        #     if sm.structure != structure:
        #         continue

        #     # print(sm.matches, sm.match_id, sm.representations)
        #     yield sm

    def set_representations(self, word_renderer):
        for _1, sm in progress(self.data.items(), "representations"):
            RepresentationAssigner.set_representations(sm, word_renderer)
            for component_id, text in sm.representations.items():
                self.db.execute("""
                    INSERT INTO Representations (colocation_id, component_id, text) 
                    VALUES (?,?,?)""", (sm.match_id, component_id, text))


    def determine_colocation_dispersions(self):
        dispersions = defaultdict(int)
        for (structure_id, *word_tups) in self.data.keys():
            for component_id, lemma in word_tups:
                dispersions[(structure_id, component_id, lemma)] += 1
        self.dispersions = dict(dispersions)
